# -*- coding: utf-8 -*-

"""
Overview:
    Purpose: Open and graph all Solar City csv files in a directory and graph
    Details:
        you need to put your solar city downloads in the same directory as this program.
        Name them so they sort in proper order from the earliest to the latest.

    Parameters:  see parameters.py
    Author:  russ_hensel, see:  http:www.opencircuits.com/User:Russ_hensel
    Download:

Environment:
    Tested in:
        OS:        Win 7
        IDE:       Spyder 2.3.1
        Language:  Python 2.7
     Should run in any Python 2.7 envoroment on any OS
     this file solar_city_plot.py

Status/history:    ** done, !! proposed

    ** working, beta forever

    ** clean up more, forever
    ** make more of app
    ** select style of graph -- in parameters
    ** add selectable smoothing -- from parameters
    ** line graph with different smoothing
    ?? add a gui for control -- no any time soon
        ?? browse to directory

"""

import sys
import matplotlib.pyplot as plt
import os
import time
import datetime
import glob
import numpy as np

# --------- in this directory ---------

import parameters
import moving_average

class SolarCityPlot:
    """
    main class for the application
    """
    def __init__( self ):
        """
        constructor as usual, sets defaults
        return nothing,
        """
        self.version         = "SolarCityPlot 2015_12_30"
        
        # start_ mostly not used 
        self.start_now       = 1
        self.start_first     = 2
        self.start_zero      = 3

        self.parameters      = parameters.Parameters( self ) # open early may effect other

        self.sec_in_day      = 60 * 60 * 24   # magic number for conversion

        self.times           = None
        self.energy_per_day  = None      # used in graphing
        
        self.plot_title      = self.parameters.plot_title

        # no just do this in parms
        self.smooth_vals     = self.parameters.smooth_vals

        self.plot_max_y      = self.parameters.plot_max_y
        
        self.plot_min_y      = 0
        self.prog_info()

    #----------------------------------------------------------------------
    def runit( self, ):
        """
        run the graphs specified in parameters.py
        """
        if self.parameters.bargraph:
            self.start_plot()
            self.graph_bar( None )
            print "show_plot?"
            self.show_plot( )

        if len( self.parameters.smooth_vals ) > 0:
            self.start_plot()
            self.graph_line(  None )
            self.show_plot( )

    #----------------------------------------------------------------------
    def logit( self, to_print ):
        """
        just a print, could be upgraded to write to a logging file
        """
        print( to_print )

    #----------------------------------------------------------------------
    def prog_info( self ):
        """
        log info about program and its argument/enviroment
        !!nice to have system time and date
        """

        self.logit( "" )
        self.logit( "============================" )
        self.logit( "" )

        self.logit( "Running " + self.version  )
        self.logit( "" )

        if len( sys.argv ) == 0:
            self.logit( "no command line arg " )
        else:
            ix_arg = 0
            for aArg in  sys.argv:

                self.logit( "command line arg " + str( ix_arg ) + " = " + sys.argv[ix_arg])
                ix_arg += 1

        self.logit( "current directory " +  os.getcwd() )
        self.logit( "==== end of prog info ========="  )
        self.logit( ""  )
        return

    # ------------------------------------------------
    def get_directory( self, path ):
        """
        get directory for csv files, now only used for
        current directory
        """
        if path == None:
             ret = os.getcwd()
        else:
             ret = path

        return ret

    # ------------------------------------------------
    def smooth( self, data_list, period ):
        """
        smooth the data using a moving average
        """
        mov_ave   = moving_average.Movingaverage( period )
        smooth_data = []
        for i_data in data_list:
            smooth_data.append( mov_ave.nextVal( i_data ) )

        return smooth_data

    # ------------------------------------------------
    def graph_line( self, path ):
        """
        graph in line style, several lines as in self.smooth_vals see parameters.py
        """
        new_path = self.get_directory(  path )

        self.files_in_path( new_path )

        x   = []
        y   = []
        self.plot_min_y    = 0

        for i_csvFile in self.csvFiles:
            ret = app.get_data_one_file( i_csvFile, self.start_now )
            #print ret
            x = x + ret[1]
            y = y + ret[0]

        self.times           = x
        self.energy_per_day  = y

        for i_smooth in self.smooth_vals:

            if i_smooth > 1:
                   smooth_energy_per_day = self.smooth( self.energy_per_day, i_smooth )
                   aa = "ave over " + str( i_smooth ) + " days"
            else:
                   smooth_energy_per_day = self.energy_per_day
                   aa = "raw data"
            #print x
            self.add_line( self.times, smooth_energy_per_day, aa, style="line" )

    # ------------------------------------------------
    def graph_bar( self, path ):
        """
        graph with no smoothing as a bar graph
        """
        new_path = self.get_directory(  path )

        self.files_in_path( new_path )

        x   = []
        y   = []
        self.plot_min_y    = 0

        for i_csvFile in self.csvFiles:
            ret = app.get_data_one_file( i_csvFile, self.start_now )
            #print ret
            x = x + ret[1]
            y = y + ret[0]

        self.add_line( x, y, "", style="bar" )

    #----------------------------------------------------------------------
    def files_in_path(self, path ):
        """
        get list of files from path of some type and sort the list
        sorting is important or the data values will be out of order
        return: none but stored in self.csvFiles
        """
        self.folderPath = path
        self.csvFiles   = glob.glob( self.folderPath + "\\*.csv" )
        self.csvFiles.sort( key=str.lower, reverse = False )
        # print( self.csvFiles )   # debug

    # ------------------------------------------------
    def get_data_one_file( self, filename, start_type ):
        """
        read one data file and extract to lists
        """
        list_str_time         = []
        list_prod             = []
        list_tot_prod         = []

        self.logit( "reading: " + filename )
        csv_data = np.genfromtxt( filename , delimiter=',', skip_header=1, skip_footer=0, dtype=None )  #names=['x', 'y', 'z'])

        for i in csv_data:
                list_str_time.append(  i[0][0:10]  )
                # https://stackoverflow.com/questions/4541155/check-if-a-number-is-int-or-float
                # this is a fix that does not really work if there is bad data in csv file
                if type(i[3]) <> np.float64:     # this seems to occur in the first spreadsheet  -- use is number better
                    print i[3]
                    print type(i[3])
                    print "not a float"
                    list_prod.append(  0.  )
                else:
                    list_prod.append(  i[3]  )

                list_tot_prod.append(  i[4]  )

        strip_fmt      = "%Y-%m-%d"
        list_timestamp = []

        if   start_type == self.start_now:
            start_adj   = self.now

        elif start_type == self.start_first:
            start_adj   = time.mktime(datetime.datetime.strptime( list_str_time[0], strip_fmt ).timetuple())

        elif start_type == self.start_zero:
            start_adj   = 0

        for i in list_str_time:

            s = i
            tstamp = time.mktime(datetime.datetime.strptime( s, strip_fmt ).timetuple())
            list_timestamp.append( ( tstamp - start_adj ) / self.sec_in_day )
            #list_timestamp.append( time.mktime(datetime.datetime.strptime(i, "%d/%m/%Y").timetuple())  )

        return ( list_prod, list_timestamp )    #x, y

    # ------------------------------------------------
    def start_plot( self,  ):
        """
        start a plot process: initialize the graph
        no data yet
        return nothing but change object state
        """
        self.lines_ix      = 0
        self.lines_ix      = 0

        self.now = time.time()

        #http:nbviewer.ipython.org/github/jrjohansson/scientific-python-lectures/blob/master/Lecture-4-Matplotlib.ipynb

        self.fig = plt.figure( figsize=(12, 12) )

        #self.axes = self.fig.add_axes([0.1, 0.1, 0.8, 0.8]) # left, bottom, width, height (range 0 to 1)
        self.axes = self.fig.add_subplot(111)  # to have multiple lines in the plot

        self.axes.set_xlabel( 'Time in days from Today' )
        self.axes.set_ylabel( 'Energy in K Watt Hr' )           # might want to make variable with type of plot

        self.axes.set_title( self.plot_title )

        return

    # ------------------------------------------------
    def add_line( self, x, y, alabel, style="line" ):
        """
        add a line of data in variables x y to the graph
        label ( legend ) alabel
        return nothing
        """
        if self.lines_ix == 0:
            if self.plot_min_y != None:
                self.axes.set_ylim([ self.plot_min_y ,  self.plot_max_y ])

        self.lines_ix      += 1
        if len( y ) <= 1:
            self.logit( "Not enough data in: " + alabel + " " + " to plot." )
            return

        avalue   = max( y )
        if avalue > self.plot_max_y:
            self.logit( "Max value in: " + alabel + " " + str( avalue ) + " beyond plot max." )

        avalue   = min( y )
        if avalue < self.plot_min_y:
            self.logit( "Mininum value in: " + alabel + " " + str( avalue ) + " below plot min." )

        if style=="bar":
            self.axes.bar(  x, y, ) # label= alabel, )
        else:
            self.axes.plot(  x, y, label= alabel, linestyle = "-", marker = "*" )

        #self.axes.plot(  x, y, label= alabel, linestyle = "--", marker = "*" )
        # http://www.sagemath.org/doc/reference/plotting/sage/plot/plot.html

        #self.axes.legend(loc=10, bbox_to_anchor=(0.5, -0.1))  # 9 is below
        self.axes.legend( loc=2 ) #  4 is lower left  2 is upper left
        return

    # ------------------------------------------------
    def show_plot( self ):   # calling show seems to destroy the plot
        """
        show the plot and !! save to file
        should supress blank plot in most cases
        return nothing
        """
        if self.lines_ix <= 0:
            return
        else:
            plt.show( self.fig )
            # save code comment out, needs a bit of fix up to work 
#            if self.save_graph[-4:] == ".png":
#                logit( "saving the file " + self.save_graph )
#                #self.fig.savefig( self.save_graph ) #or use fig.savefig("filename.png", dpi=200)
#                pass
#            self.fig      = None  # cannot us show() again on same plot, use None as a flag
#            self.lines_ix = 0

        return

# ------------------------------------------------
if __name__ == '__main__':
    """
    run the application as an object
    """
    app   = SolarCityPlot( )
    app.runit()

# ====================== eof ========================






